@@ -122,6 +122,7 @@ DEFINE_TEST(test_compat_solaris_pax_sparse)
 DEFINE_TEST(test_compat_solaris_tar_acl)
 DEFINE_TEST(test_compat_star_acl_posix1e)
 DEFINE_TEST(test_compat_star_acl_nfs4)
+DEFINE_TEST(test_compat_tar_directory)
 DEFINE_TEST(test_compat_tar_hardlink)
 DEFINE_TEST(test_compat_uudecode)
 DEFINE_TEST(test_compat_uudecode_large)
@@ -337,13 +338,18 @@ DEFINE_TEST(test_read_format_rar5_distance_overflow)
 DEFINE_TEST(test_read_format_rar5_nonempty_dir_stream)
 DEFINE_TEST(test_read_format_rar5_fileattr)
 DEFINE_TEST(test_read_format_rar5_different_window_size)
+DEFINE_TEST(test_read_format_rar5_window_buf_and_size_desync)
 DEFINE_TEST(test_read_format_rar5_arm_filter_on_window_boundary)
 DEFINE_TEST(test_read_format_rar5_different_solid_window_size)
 DEFINE_TEST(test_read_format_rar5_different_winsize_on_merge)
 DEFINE_TEST(test_read_format_rar5_block_size_is_too_small)
+DEFINE_TEST(test_read_format_rar5_sfx)
+DEFINE_TEST(test_read_format_rar5_decode_number_out_of_bounds_read)
+DEFINE_TEST(test_read_format_rar5_bad_window_size_in_multiarchive_file)
 DEFINE_TEST(test_read_format_rar_encryption_data)
 DEFINE_TEST(test_read_format_rar_encryption_header)
 DEFINE_TEST(test_read_format_rar_encryption_partially)
+DEFINE_TEST(test_read_format_rar_filter)
 DEFINE_TEST(test_read_format_rar_invalid1)
 DEFINE_TEST(test_read_format_raw)
 DEFINE_TEST(test_read_format_tar)
@@ -373,12 +379,20 @@ DEFINE_TEST(test_read_format_zip_bzip2_one_file)
 DEFINE_TEST(test_read_format_zip_bzip2_one_file_blockread)
 DEFINE_TEST(test_read_format_zip_bzip2_multi)
 DEFINE_TEST(test_read_format_zip_bzip2_multi_blockread)
+DEFINE_TEST(test_read_format_zip_zstd_one_file)
+DEFINE_TEST(test_read_format_zip_zstd_one_file_blockread)
+DEFINE_TEST(test_read_format_zip_zstd_multi)
+DEFINE_TEST(test_read_format_zip_zstd_multi_blockread)
 DEFINE_TEST(test_read_format_zip_xz_multi)
 DEFINE_TEST(test_read_format_zip_xz_multi_blockread)
 DEFINE_TEST(test_read_format_zip_ppmd8_crash_1)
 DEFINE_TEST(test_read_format_zip_bz2_hang_on_invalid)
 DEFINE_TEST(test_read_format_zip_ppmd8_crash_2)
 DEFINE_TEST(test_read_format_zip_lzma_alone_leak)
+DEFINE_TEST(test_read_format_zip_lzma_stream_end)
+DEFINE_TEST(test_read_format_zip_lzma_stream_end_blockread)
+DEFINE_TEST(test_read_format_zip_7z_lzma)
+DEFINE_TEST(test_read_format_zip_7z_deflate)
 DEFINE_TEST(test_read_format_zip_utf8_paths)
 DEFINE_TEST(test_read_format_zip_comment_stored)
 DEFINE_TEST(test_read_format_zip_encryption_data)
@@ -424,8 +438,9 @@ DEFINE_TEST(test_read_format_zip_with_invalid_traditional_eocd)
 DEFINE_TEST(test_read_format_zip_zip64a)
 DEFINE_TEST(test_read_format_zip_zip64b)
 DEFINE_TEST(test_read_large)
-DEFINE_TEST(test_schily_xattr_pax)
 DEFINE_TEST(test_read_pax_truncated)
+DEFINE_TEST(test_read_pax_xattr_rht_security_selinux)
+DEFINE_TEST(test_read_pax_xattr_schily)
 DEFINE_TEST(test_read_position)
 DEFINE_TEST(test_read_set_format)
 DEFINE_TEST(test_read_set_wrong_format)
@@ -442,6 +457,7 @@ DEFINE_TEST(test_read_truncated_filter_lzip)
 DEFINE_TEST(test_read_truncated_filter_lzma)
 DEFINE_TEST(test_read_truncated_filter_lzop)
 DEFINE_TEST(test_read_truncated_filter_xz)
+DEFINE_TEST(test_short_writes)
 DEFINE_TEST(test_sparse_basic)
 DEFINE_TEST(test_fully_sparse_files)
 DEFINE_TEST(test_tar_filenames)
@@ -460,6 +476,7 @@ DEFINE_TEST(test_warn_missing_hardlink_target)
 DEFINE_TEST(test_write_disk)
 DEFINE_TEST(test_write_disk_appledouble)
 DEFINE_TEST(test_write_disk_failures)
+DEFINE_TEST(test_write_disk_fixup)
 DEFINE_TEST(test_write_disk_hardlink)
 DEFINE_TEST(test_write_disk_hfs_compression)
 DEFINE_TEST(test_write_disk_lookup)
@@ -551,6 +568,7 @@ DEFINE_TEST(test_write_format_zip_winzip_aes256_encryption)
 DEFINE_TEST(test_write_format_zip_compression_store)
 DEFINE_TEST(test_write_format_zip_empty)
 DEFINE_TEST(test_write_format_zip_empty_zip64)
+DEFINE_TEST(test_write_format_zip_size_unset)
 DEFINE_TEST(test_write_format_zip_file)
 DEFINE_TEST(test_write_format_zip_file_zip64)
 DEFINE_TEST(test_write_format_zip_large)
